#!/usr/bin/php
<?php
/**
 * Copyright (C) 2010 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
	define("EOL" , "\n");
	// enable server/client debug messages	
	define("DEBUG" , false);
	
	/**
	 * Open a server connection to TS3 server
	 * 
	 * @param string $host
	 * @param int $port
	 * @param string $errno
	 * @param string $errstr
	 * @param int $timeout
	 */
	function Open($host, $port, &$errno, &$errstr, $timeout) 
	{
		$socket = fsockopen($host, $port, $errno, $errstr, $timeout);
		if(!$socket or fread($socket, 4) != "TS3\n") {
			die("ERR: Teamspeak server not running!" . EOL);
			return false;
		}
		fread($socket, 200);
		return $socket;
	}
	
	/**
	 * Select the virtualserver on opened TS3 server connection
	 * 
	 * @param handle $socket
	 * @param int $serverid
	 */
	function selectServerId($socket, $serverid)
	{
		$command = "use sid={$serverid}";
		if( DEBUG ) echo ' >> ' . $command . EOL;
		fputs($socket, $command . EOL);
		if ( ($msg = Expected($socket,'ok')) !== TRUE ) {
			die('ERR: ' . ErrMessage($msg) . EOL);
		}
	}
	
	/**
	 * Returns and array containing serverinfo variables
	 * note: that variables are different if you connect to localhost or a worldwide server
	 * 
	 * @param handle $socket
	 */
	function serverInfo($socket) 
	{
		$command = 'serverinfo';
		if( DEBUG ) echo ' >> ' . $command . EOL;
		fputs($socket, $command . EOL);
		return serverInfo2Array(Read($socket));
	}
	// private use is easy to debug 
	function serverInfo2Array($sinfo) 
	{
		$out = explode(" ", $sinfo);
		$result = array();
		foreach($out as $line){
			if( DEBUG ) echo ' s: ' . $line . EOL;
			@list($k,$v) = explode("=", $line);
			$result[$k] = str_replace('\s'," ", $v);
		}
		return $result;
	}	
	
	/**
	 * Perform login on remote TS3 Server
	 * 
	 * @param handle $socket
	 * @param handle $user
	 * @param handle $password
	 */	
	function Login($socket, $user, $password) 
	{
		$command = "login client_login_name={$user} client_login_password={$password}";
		if( DEBUG ) echo ' >> ' . $command . EOL;
		fputs($socket,  $command . EOL);
		if ( ($msg = Expected($socket,'ok')) !== TRUE ) {
			die('ERR: ' . ErrMessage($msg) . EOL);
		}
	}

	/**
	 * Check if the string contains at its end a message
	 * 
	 * @param handle $socket
	 * @param string $msg
	 */		
	function Expected($socket, $msg )
	{
		$out = trim(fread($socket, 10000));
		if( DEBUG ) echo ' << ' . $out . EOL;
		$pattern = '/' . $msg .'$/';
		if ( preg_match('/' . $msg .'$/', $out ) ) {
			return TRUE;
		} else {
			return $out;
		}
		return $out;
	}
	
	/**
	 * Simply read from socket
	 * 
	 * @param handle $socket
	 */
	function Read($socket)
	{
		$out = trim(fread($socket, 10000));
		if( DEBUG ) echo ' << ' . $out . EOL;
		return 	$out;
	}
	
	/**
	 * Close a server connection to TS3 server
	 * 
	 * @param handle $socket
	 */
	function Close($socket) 
	{
		$command = 'quit';
		if( DEBUG ) echo ' >> ' . $out . EOL;
		fputs($socket, $command . EOL);
		fclose($socket);
	}
	/**
	 * Parse message from server, ONLY message not even extra_msg
	 * 
	 * @param string $string
	 */	
	function ErrMessage($string)
	{
		$err = preg_match('/msg=(.*) extra_msg=(.*)/', $string, $out);
		return str_replace('\s'," ", $out[1]);
	}

// usage
if ( count($argv) < 6 ) {
	echo "Usage: {$argv[0]} <ip> <sid> <user> <pass> < clients | bandwidth >" . EOL;
	exit;
}
// open
$errstr = $errno = '';
$socket = Open($argv[1], 10011, $errno, $errstr, 1);
if($errno != 0) exit(1);
// select server id
selectServerId($socket, $argv[2]);
// login
Login($socket, $argv[3], $argv[4]);
// fetch server infoes
$out = serverInfo($socket);
// parse argv[5]
switch($argv[5]) {
	case 'clients':
		$max = intval(@$out['virtualserver_maxclients']);
		$now = intval(@$out['virtualserver_clientsonline']) - 1; // server count us as a client so -1
		echo "max:{$max} now:{$now}";
		break;
	case 'bandwidth':
		$in = intval(@$out['connection_bandwidth_received_last_minute_total']) ;
		$out = intval(@$out['connection_bandwidth_sent_last_minute_total']) ;
		echo "band_out:{$out} band_in:{$in}";
		break;
}
